module slave_addr_decoder #(
    parameter ADDR_MAP_FILE = "addr_map.mem",

    parameter N_TARGETS = 2,
    parameter AXI_ADDR_WIDTH = 32
)(
    input [AXI_ADDR_WIDTH-1:0] global_addr,
    output logic [AXI_ADDR_WIDTH-1:0] local_addr,

    input master_valid,
    output logic [$clog2(N_TARGETS)-1:0] target_sel,
    output logic target_valid
);

logic [AXI_ADDR_WIDTH-1:0] ADDR_MAP [N_TARGETS * 2];

initial begin
    // maybe we can readmemh to get the mappings parametricaly?
    $readmemh(ADDR_MAP_FILE, ADDR_MAP);

    ADDR_MAP[0] = 32'h00000000; 
    ADDR_MAP[1] = 32'h000001ff;
    ADDR_MAP[2] = 32'h0000ff00;
    ADDR_MAP[3] = 32'h0000ffff;
end


always_comb begin
    target_valid = 0;
    target_sel = 0;
    local_addr = '0;
    for (int i = 0; i < N_TARGETS; i++) begin
        if (global_addr >= ADDR_MAP[2*i] && global_addr <= ADDR_MAP[2*i+1]) begin
            target_sel = i;
            target_valid = master_valid;
            local_addr = global_addr - ADDR_MAP[2*i];
        end
    end
end

endmodule